#ifndef _TIMESTMP_H
#define _TIMESTMP_H



#include "..\types\types.h"         // typedef Bit32, class Bit64

#define CLOCK_RATE          PentiumClock::clock_rate
#define PC_TICK_RATE        18.206482
#define BITS_TO_KEEP        (32 - BITS_TO_TOSS)
#ifdef CLK_86  // Use 80x86 clock hardware (rather than Pentium)
#define BITS_TO_TOSS        4
#else
#define BITS_TO_TOSS        22   // how many bits of lsb to throw away?
#endif



// 64-bit value containing a count of processor
// clock ticks since the processor was powered on
//
// unknown whether it resets when processor is reset
//
// with a 120 MHz clock, wraps every 117K years
//        166                         84K
//
// if we only look at 32 bits starting 22 bits from
// least significant one, we get precision of 35 mS
// with 120 MHz clock (25 mS with 166 MHz clock) and
// it will wrap every 114 years with 120 MHz clock
// (82.5 years with 166 MHz clock)
//
class PentiumClock : public Bit64
{

public:
    static double clock_rate;
    static double init();

    // no destructor needed

    // construct w/o args
    //
    PentiumClock()
    {
    }

#if 1
    // construct from a Bit64
    //
    PentiumClock( const Bit64 &second )
    :
        Bit64( second )
    {
    }
#endif

    // construct from double-precision floating
    // point using that assignment operator
    //
    PentiumClock( double second )
    :
        Bit64( second )
    {
    }

    // fill me in
    //
    void set( void );

    // assignment from seconds
    //
    void put_seconds( const double second )
    {
        *this = second * CLOCK_RATE;
    }

    // assignment from PC timer clock ticks
    //
    void put_ticks( const Bit32 ticks )
    {
        put_seconds( ticks / PC_TICK_RATE );
    }

    // assignment from a 32 bit truncation
    //
    void put_trunc32( const Bit32 truncation )
    {
        low = truncation << BITS_TO_TOSS;
//        high = (high & ((~0) << BITS_TO_KEEP))  /* Nevil, 31 Dec 96 */
//             | (truncation >> BITS_TO_KEEP);
        high = truncation >> BITS_TO_KEEP;
    }

    // conversion to seconds
    //
    double get_seconds() const
    {
        return ((double) *this) / CLOCK_RATE;
    }

    // conversion to PC timer clock ticks
    //
    Bit32 get_ticks() const
    {
        return get_seconds() * PC_TICK_RATE;
    }

    // truncation to 32 bit integer
    //
    Bit32 get_trunc32() const
    {
        return (high << BITS_TO_KEEP) | (low >> BITS_TO_TOSS);
    }          // Nevil, 31 Dec 96

    // busy-waits the number of CPU clocks you specify
    // and tells you how many iterations it did
    //
    static Bit32 wait( Bit32 count );

   // Read clock in 'implementation' units  (Nevil, 2 Jul 97)
   //
   double get_native() const
   {
      return (double) *this;
      }

};

extern "C" void hwt_init(void);  /* Initialise timer routines */
extern "C" void hwt_fin(void);   /* Shut down timer routines */

#endif          // not included yet
