#include <stdio.h>                  // printf()
#include <stdlib.h>                 // atoi()
#include <stdarg.h>                 // va_start() va_end() va_list
#include <string.h>                 // stricmp()
#include <mem.h>                    // memset()
#include <errno.h>                  // errno _sys_errlist
#include <conio.h>                  // kbhit()
#include "..\intel\timestmp.h"
#include "..\wattcp\tcp.h"

void init_trace(int level);

#define CONTROL_WAIT             100    // measured in seconds
#define FILENAME_SIZE            256
#define MODE_SIZE                100
#define BUFFER_SIZE              (1024*1024)

#define TRUE                       1
#define FALSE                      0



char buffer[ BUFFER_SIZE ];



// wait for data to be available
// or socket to close
//
void wait_for_data( udp_Socket *sock )
{
    while( tcp_tick( sock ) && !sock_dataready( sock ) )
        ;
}



int cleanup( int retcode, udp_Socket *sock, const char *format, ... )
{
    va_list argptr;

    va_start( argptr, format );
    vprintf( format, argptr );
    va_end( argptr );

    // signal that we want socket closed
    //
    sock_close( sock );

    // return to OS
    //
    return retcode;
}



int sock_to_file( udp_Socket *sock, const char *filename, long *bytes )
{
    int result, result2;
    FILE *file_handle;

    // log to the console
    //
    printf( "copying TCP/IP stream to file '%s'\n", filename );

    // no bytes RX yet
    //
    *bytes = 0;

    // open up the file locally
    //
    file_handle = fopen( filename, "wb" );
    if( file_handle == NULL )
    {
        return cleanup( 10, sock, "cannot create file '%s'\n", filename );
    }

    // change socket to binary mode so all bytes get copied verbatin
    //
    sock_mode( sock, TCP_MODE_BINARY );

    // while client hasn't closed socket...
    //
    while( tcp_tick( sock ) )
    {
        // stop if the user pressed a key
        //
        if( kbhit() )
            return cleanup( 11, sock, "\nhalted by keypress\n" );

        // read up to one buffer's worth of data
        //
        result = sock_fastread( sock, buffer, BUFFER_SIZE );

        // if we got anything...
        //
        if( result > 0 )
        {
            // write it to disk file
            //
            result2 = fwrite( buffer, 1, result, file_handle );
            if( result2 != result )
            {
                fclose( file_handle );
                return cleanup( 12, sock, "could only write %d of %d bytes to file '%s'\n", result2, result, filename );
            }

            // keep track of how many bytes we have done
            //
            *bytes += result;
        }
    }

    // close the disk file
    //
    fclose( file_handle );

    // exit to OS successfully
    //
    return 0;
}



int file_to_sock( udp_Socket *sock, const char *filename, long *bytes )
{
    int result, result2;
    FILE *file_handle;

    // log to the console
    //
    printf( "copying file '%s' to TCP/IP stream\n", filename );

    // no bytes TX yet
    //
    *bytes = 0;

    // open up the file locally
    //
    file_handle = fopen( filename, "rb" );
    if( file_handle == NULL )
    {
        return cleanup( 20, sock, "error %d '%s' opening existing file '%s'\n", errno, _sys_errlist[errno], filename );
    }

    // change socket to binary mode so all bytes get copied verbatin
    //
    sock_mode( sock, TCP_MODE_BINARY );

    // while client hasn't closed socket
    // and we haven't reached end of file...
    //
    while( tcp_tick( sock ) && !feof( file_handle ) )
    {
        // stop if the user pressed a key
        //
        if( kbhit() )
            return cleanup( 21, sock, "\nhalted by keypress\n" );

        // read up to one buffer's worth of data
        //
        result = fread( buffer, 1, BUFFER_SIZE, file_handle );

        // if we got anything...
        //
        if( result > 0 )
        {
            // write it to socket
            //
            result2 = sock_write( sock, buffer, result );
            if( result2 != result )
            {
                fclose( file_handle );
                return cleanup( 22, sock, "could only write %d of %d bytes to socket\n", result2, result );
            }

            // keep track of how many bytes we have done
            //
            *bytes += result;
        }
    }

    // close the disk file
    //
    fclose( file_handle );

    // close the socket ourselves and
    // exit to OS successfully
    //
    return cleanup( 0, sock, "" );
}



int client( udp_Socket *sock, unsigned port, char *hostname, const char *mode_string, char *filename )
{
    int result;
    int mode_is_upload;
    longword hostaddr;
    char *command;
    PentiumClock start,now;
    double s;
    long bytes;

    // resolve the host name to an address
    //
    hostaddr = resolve( hostname );
    if( !hostaddr )
    {
        printf( "unable to resolve host name '%s' to numerical IP address\n", hostname );
        return 40;
    }

    // convert the mode into a command
    //
    if( !stricmp( mode_string, "u" ) )
    {
        mode_is_upload = TRUE;
        command = "SAVE";
    }
    else if( !stricmp( mode_string, "d" ) )
    {
        mode_is_upload = FALSE;
        command = "RETR";
    }
    else
    {
        printf( "invalid file transfer mode '%s'\n", mode_string );
        return 41;
    }

    // initiate a connection
    //
    result = udp_open( sock, 0, hostaddr, port, NULL );
    if( result == 0 )
    {
        printf( "cannot open UDP port %d\nis host responding to ARP?\n", port );
        return 42;
    }

    // wait for our connection to finish
    //
    //sock_wait_established( sock, CONTROL_WAIT, NULL, &result );
    if( !mode_is_upload )
        wait_for_data( sock );

    // remember when data transfer started
    //
    start.set();

    if( mode_is_upload )
        result = file_to_sock( sock, filename, &bytes );
    else
        result = sock_to_file( sock, filename, &bytes );

    // calculate how much time has elapsed
    //
    now.set();
    now -= start;
    s = now.get_seconds();

    // inform the user how much we got and how fast
    //
    printf( "%ld bytes, %.3f seconds,  %.3f KB/s\n",
       bytes, s, ((double)bytes/1000.0)/s );

    return result;

sock_err:
    printf( "got error %d from sock_wait_established\n", result );
    return 999;
}



int main( int argc, char *argv[] )
{
    const char *program = argv[0];
    unsigned control_port;
    int result;
    udp_Socket control_socket;

    if( argc!=5 )
    {
        printf( "receive:  %s udp-port-number hostname u filename\n", program );
        printf( "transmit: %s udp-port-number hostname d filename\n", program );
        return 1;
    }

    control_port = atoi( argv[ 1 ] );

    // initialize TCP/IP library
    //
    sock_init();

//   tcp_set_debug_state(1);  /* Display flags for each packet */
//   tcp_set_debug_state(2);  /* Display full trace data */
//   init_trace(2);  /* Nevil's tcp packet trace - full */
//   init_trace(1);  /* Nevil's tcp packet trace - times only */

    // set socket to known state
    //
    memset( &control_socket, 0, sizeof(control_socket) );

    return client( &control_socket, control_port, argv[2], argv[3], argv[4] );
}



// written 7/17/96 by Joel Apisdorf
//
